package com.persagy.account.controller;

import java.io.UnsupportedEncodingException;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.apache.commons.collections4.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.persagy.account.config.ApplicationProperties;
import com.persagy.account.pojo.vo.thirty.SaasCodeTokenVO;
import com.persagy.account.pojo.vo.thirty.SaasCodeVO;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;
import com.persagy.security.util.BouncycastleCipher;
import com.persagy.security.util.SecureAES;

import cn.hutool.core.util.BooleanUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.crypto.digest.DigestAlgorithm;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 三方授权中心
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年4月27日 下午3:08:16
 */
@RestController
@Api(tags = "授权管理")
@RequestMapping(value = "/authCenter", method = RequestMethod.POST)
public class SaasAuthCenterController {
	
	private static final String SAAS_CLIENT_ID_REDIS_KEY = "SAAS_PLATFORM_CLIENT_ID_KEY";
	
	@Autowired
	private ApplicationProperties properties;
	
    @Autowired
    private BouncycastleCipher bouncycastleCipher;
    
	@Autowired
	private RedisTemplate<String, Object> redisTemplate;
	
    /**
     * 申请授权码
     */
    @ApiOperation(value = "申请授权码")
    @RequestMapping(value = "code")
    public ResponseResult getCode(@RequestBody @Validated SaasCodeVO model) {
    	// 验证客户端ID是否存在
    	Boolean member = this.redisTemplate.opsForSet().isMember(SAAS_CLIENT_ID_REDIS_KEY, model.getClientId());
    	if (!BooleanUtil.isTrue(member)) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0001.getCode(), ResponseCode.A0001.getDesc());
		}
    	// 验证是否授权码已存在
    	String fastSimpleUUID = IdUtil.fastSimpleUUID();
    	Boolean setIfAbsent = this.redisTemplate.opsForValue().setIfAbsent(model.getClientId() + "_CODE", fastSimpleUUID, 60000L, TimeUnit.MILLISECONDS);
    	if (!BooleanUtil.isTrue(setIfAbsent)) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0302.getCode(), ResponseCode.A0001.getDesc());
		}
    	Map<String, String> code = new HashedMap<String, String>();
    	code.put("code", fastSimpleUUID);
    	return ResponseResultUtil.successResult(code);
    }
    
    /**
     * 根据授权码获取token
     * @throws UnsupportedEncodingException 
     */
    @ApiOperation(value = "根据授权码获取token")
    @RequestMapping(value = "codeToken")
    public ResponseResult getCodeToken(@RequestBody @Validated SaasCodeTokenVO model) throws UnsupportedEncodingException {
    	// 验证客户端ID是否存在
    	Boolean member = this.redisTemplate.opsForSet().isMember(SAAS_CLIENT_ID_REDIS_KEY, model.getClientId());
    	if (!BooleanUtil.isTrue(member)) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0001.getCode(), ResponseCode.A0001.getDesc());
		}
    	// 验证是否授权码存在
    	Object object = this.redisTemplate.opsForValue().get(model.getClientId() + "_CODE");
    	if (object == null) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0311.getCode(), ResponseCode.A0311.getDesc());
		}
    	String code = (String) object;
    	if (!model.getCode().equals(code)) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0303.getCode(), ResponseCode.A0303.getDesc());
		}
    	// 生成token
    	String key = model.getClientId() + "_TOKEN";
    	long expire = Long.parseLong(this.properties.getTokenExpire());
    	String sign = this.bouncycastleCipher.encrypt(this.tokenInfo(model.getClientId(), code), key, DigestAlgorithm.MD5, expire, TimeUnit.MINUTES);
    	SecureAES aes = new SecureAES(properties.getAesKey(), properties.getAesIv());
    	String data = aes.encryptAccount(this.tokenAuthInfo(key));
    	String token = data + SaasCommonConstant.POINT_JOIN_SYMBOL + sign;
    	Map<String, Object> map = new HashedMap<String, Object>();
    	map.put("token", token);
    	map.put("expire", expire);
    	return ResponseResultUtil.successResult(map);
    }
    
    /**
     * 
     * @param loginApp
     * @return
     */
    private JSONObject tokenAuthInfo(String key) {
		JSONObject object = new JSONObject();
		object.put(SaasCommonConstant.ACCOUNT_ID, key);
		return object;
	}
    
    /**
     * 根据授权码组合token原信息
     * @param clientId
     * @param code
     * @return
     */
    private String tokenInfo(String clientId, String code) {
		JSONObject object = new JSONObject();
		object.put("clientId", clientId);
		object.put("date", System.currentTimeMillis());
		object.put("code", code);
		return object.toJSONString();
	}
    
}
